@@ -77,6 +77,11 @@ APR_DECLARE(apr_status_t) apr_shm_remove(const char *filename,
     return APR_ENOTIMPL;
 }
 
+APR_DECLARE(apr_status_t) apr_shm_delete(apr_shm_t *m)
+{
+    return APR_ENOTIMPL;
+} 
+
 APR_DECLARE(apr_status_t) apr_shm_attach(apr_shm_t **m,
                                          const char *filename,
                                          apr_pool_t *pool)
@@ -130,6 +135,8 @@ APR_DECLARE(apr_size_t) apr_shm_size_get(const apr_shm_t *m)
     return size;
 }
 
+APR_PERMS_SET_ENOTIMPL(shm)
+
 APR_POOL_IMPLEMENT_ACCESSOR(shm)
 
 APR_DECLARE(apr_status_t) apr_os_shm_get(apr_os_shm_t *osshm,
